//
//  MGCopyAnswerTableViewController.m
//  MGCopyAnswerTable
//
//  Created by Alexandre Colucci on 21.01.2016.
//  Copyright © 2016 Alexandre Colucci. All rights reserved.
//

#import "MGCopyAnswerTableViewController.h"
#include "MGDatabase.h"
#import <dlfcn.h>
#import "MGTableCell.h"

#define COPYANSWERARRAY_NAME_KEY		@"key"
#define COPYANSWERARRAY_OBFUSCATED_KEY	@"obfuscated"
#define COPYANSWERARRAY_VALUE_KEY		@"value"

#define CELL_REUSE_IDENTIFIER			@"CellReuseIdentifier"

#pragma mark - Find MGCopyAnswer

id (*MGCopyAnswer)(NSString *) = NULL;


/**
 Find the MGCopyAnswer function in libMobileGestalt.dylib
 */
void findMGCopyAnswer()
{
	void *libMobileGestalt = dlopen("/usr/lib/libMobileGestalt.dylib", RTLD_LAZY);
	if(libMobileGestalt == NULL)
	{
		NSLog(@"Could not find libMobileGestalt!");
		return;
	}
	
	MGCopyAnswer = dlsym(libMobileGestalt, "MGCopyAnswer");
	if(MGCopyAnswer == NULL)
	{
		NSLog(@"Could not find MGCopyAnswer!");
		return;
	}
	
	NSLog(@"Found MGCopyAnswer %p", MGCopyAnswer);
}


@interface MGCopyAnswerTableViewController ()

@property (strong, nonatomic) NSArray <NSDictionary *> *mgCopyAnswerArray;
@property (strong, nonatomic) NSArray <NSDictionary *> *mgCopyAnswerNullArray;

@end

@implementation MGCopyAnswerTableViewController


- (instancetype)init
{
    self = [super initWithStyle:UITableViewStylePlain];
    if (self)
	{
		findMGCopyAnswer();
		[self recreateMGCopyAnswerArrays];
    }
	
    return self;
}

- (NSArray *)sortArrayByName:(NSArray *)inArray
{
	return [inArray sortedArrayUsingComparator:^NSComparisonResult(NSDictionary* dict1, NSDictionary* dict2)
	{
		NSString *key1 = [dict1 objectForKey:COPYANSWERARRAY_NAME_KEY];
		NSString *key2 = [dict2 objectForKey:COPYANSWERARRAY_NAME_KEY];
		
		if(key1 == nil && key2 != nil)
		{
			return NSOrderedDescending;
		}
		else if(key1 != nil && key2 == nil)
		{
			return NSOrderedAscending;
		}
		else if([key1 isEqualToString:key2] || (key1 == nil && key2 == nil))
		{
			NSString *obfuscatedKey1 = [dict1 objectForKey:COPYANSWERARRAY_OBFUSCATED_KEY];
			NSString *obfuscatedKey2 = [dict2 objectForKey:COPYANSWERARRAY_OBFUSCATED_KEY];
			return [obfuscatedKey1 compare:obfuscatedKey2 options:(NSCaseInsensitiveSearch | NSNumericSearch | NSDiacriticInsensitiveSearch)];
		}
		else
		{
			return [key1 compare:key2 options:(NSCaseInsensitiveSearch | NSNumericSearch | NSDiacriticInsensitiveSearch)];
		}
	}];
}

- (void)recreateMGCopyAnswerArrays
{
	NSMutableArray *mgCopyAnswerArray = [[NSMutableArray alloc] init];
	NSMutableArray *mgCopyAnswerNullArray = [[NSMutableArray alloc] init];
	
	int keyIndex = 0;
	while(true)
	{
		struct tKeyMapping keyMapping = keyMappingTable[keyIndex];
		if(keyMapping.obfuscatedKey == NULL)
			break;
		
		NSString *obfuscatedKey = [NSString stringWithUTF8String:keyMapping.obfuscatedKey];
		id value = MGCopyAnswer(obfuscatedKey);
		
		NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
		[dict setObject:obfuscatedKey forKey:COPYANSWERARRAY_OBFUSCATED_KEY];
		
		if(value != nil)
		{
			[dict setObject:value forKey:COPYANSWERARRAY_VALUE_KEY];
		}
		
		if(keyMapping.key != nil)
		{
			NSString *key = [NSString stringWithUTF8String:keyMapping.key];
			[dict setObject:key forKey:COPYANSWERARRAY_NAME_KEY];
		}
		
		if(value != nil)
		{
			[mgCopyAnswerArray addObject:dict];
		}
		else
		{
			[mgCopyAnswerNullArray addObject:dict];
		}
		
		keyIndex++;
	}
	
	
	// Sort by name
	self.mgCopyAnswerArray = [self sortArrayByName:mgCopyAnswerArray];
	self.mgCopyAnswerNullArray = [self sortArrayByName:mgCopyAnswerNullArray];
}


- (void)viewDidLoad
{
	[super viewDidLoad];
	
	[self.tableView registerNib:[UINib nibWithNibName:@"MGTableCell" bundle:nil] forCellReuseIdentifier:CELL_REUSE_IDENTIFIER];
	self.tableView.estimatedRowHeight = 45.0;
	
	self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Reload" style:UIBarButtonItemStylePlain target:self action:@selector(reload)];
}

- (void)reload
{
	[self recreateMGCopyAnswerArrays];
	[self.tableView reloadData];
}

-(NSArray *)mgCopyAnswerArrayForSection:(NSInteger)section
{
	if(section == 0)
	{
		return self.mgCopyAnswerArray;
	}
	else if(section == 1)
	{
		return self.mgCopyAnswerNullArray;
	}
	
	return nil;
}

#pragma mark - Table View Datasource


-(NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
	return 2;
}

-(NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	return [[self mgCopyAnswerArrayForSection:section] count];
}

-(UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	MGTableCell* cell = (MGTableCell*)[tableView dequeueReusableCellWithIdentifier:CELL_REUSE_IDENTIFIER];

	if (cell == nil)
	{
		cell = [[MGTableCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CELL_REUSE_IDENTIFIER];
	}
	
	NSArray *mgCopyAnswerArray = [self mgCopyAnswerArrayForSection:indexPath.section];
	NSDictionary *dict = nil;
	if(indexPath.row < [mgCopyAnswerArray count])
	{
		dict = mgCopyAnswerArray[indexPath.row];
	}
		
	NSString *obfuscatedKey = dict[COPYANSWERARRAY_OBFUSCATED_KEY];
	cell.obfuscatedKeyLabel.text = [obfuscatedKey length] > 0 ? obfuscatedKey : @"(UNKNOWN)";
	
	id value = dict[COPYANSWERARRAY_VALUE_KEY];
	NSString *stringValue = [value description];
	cell.valueLabel.text = stringValue != nil ? stringValue : nil;
	
	NSString *name = dict[COPYANSWERARRAY_NAME_KEY];
	cell.keyLabel.text = [name length] > 0 ? name : @"(UNKNOWN)";
	
	return cell;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
	if(section == 0)
	{
		return [NSString stringWithFormat:@"Keys with valid value (%ld)", (long)[self tableView:tableView numberOfRowsInSection:0]];
	}
	else if(section == 1)
	{
		return [NSString stringWithFormat:@"Keys with no value (%ld)", (long)[self tableView:tableView numberOfRowsInSection:1]];
	}
	
	return nil;
}

@end
